"""empty-comment test-case"""
A = 5  #
#
A = '#' + '1'
print(A)  #
print("A=", A)  # should not be an error#
A = "#pe\0ace#love#"  #
A = "peace#love"  # \0 peace'#'''' love#peace'''-'#love'-"peace#love"#
#######
