""" The sister file of similar_lines_a, another file designed to have lines of
similarity when compared to its sister file

As with the sister file, we use lorm-ipsum to generate 'random' code. """
# Copyright (c) 2020 Frank Harrison <frank@doublethefish.com>


class Nulla:
    tortor = "ultrices quis porta in"
    sagittis = "ut tellus"

    def pulvinar(self, blandit, metus):
        egestas = [mauris for mauris in zip(blandit, metus)]
        neque = (egestas, blandit)

    def similar_function_5_lines(self, similar):  # line same #1
        some_var = 10  # line same #2
        someother_var *= 300  # line same #3
        fusce = "sit"  # line same #4
        amet = "tortor"  # line same #5
        iaculis = "dolor"  # line diff
        return some_var, someother_var, fusce, amet, iaculis, iaculis  # line diff


def tortor(self):
    ultrices = 2
    quis = ultricies * "porta"
    return ultricies, quis


class Commodo:
    def similar_function_3_lines(self, tellus):  # line same #1
        agittis = 10  # line same #2
        tellus *= 300  # line same #3
        laoreet = "commodo "  # line diff
        return agittis, tellus, laoreet  # line diff
