[Messages Control]
disable=too-few-public-methods,print-statement

[typecheck]
generated-members=
    \Afunctional\.g\.generated_members\.Klass\.ham\Z,
    DoesNotExist,
    "[a-zA-Z]+_set{1,2}",
    node_classes.Tuple.*,
    checkers.*?base,
    (session|SESSION).rollback
