""" A file designed to have lines of similarity when compared to similar_lines_b

We use lorm-ipsum to generate 'random' code. """
# Copyright (c) 2020 Frank Harrison <frank@doublethefish.com>


def adipiscing(elit):
    etiam = "id"
    dictum = "purus,"
    vitae = "pretium"
    neque = "Vivamus"
    nec = "ornare"
    tortor = "sit"
    return etiam, dictum, vitae, neque, nec, tortor


class Amet:
    def similar_function_3_lines(self, tellus):  # line same #1
        agittis = 10  # line same #2
        tellus *= 300  # line same #3
        return agittis, tellus  # line diff

    def lorem(self, ipsum):
        dolor = "sit"
        amet = "consectetur"
        return (lorem, dolor, amet)

    def similar_function_5_lines(self, similar):  # line same #1
        some_var = 10  # line same #2
        someother_var *= 300  # line same #3
        fusce = "sit"  # line same #4
        amet = "tortor"  # line same #5
        return some_var, someother_var, fusce, amet  # line diff

    def __init__(self, moleskie, lectus="Mauris", ac="pellentesque"):
        metus = "ut"
        lobortis = "urna."
        Integer = "nisl"
        (mauris,) = "interdum"
        non = "odio"
        semper = "aliquam"
        malesuada = "nunc."
        iaculis = "dolor"
        facilisis = "ultrices"
        vitae = "ut."

        return (
            metus,
            lobortis,
            Integer,
            mauris,
            non,
            semper,
            malesuada,
            iaculis,
            facilisis,
            vitae,
        )

    def similar_function_3_lines(self, tellus):  # line same #1
        agittis = 10  # line same #2
        tellus *= 300  # line same #3
        return agittis, tellus  # line diff
